@@ -1,28 +1,41 @@
-# Copyright (C) 2005, 2006 Martin von Löwis
-# Licensed to PSF under a Contributor Agreement.
-"""
-Implements the bdist_msi command.
+"""Implements the bdist_msi command.
 
 Borrowed from distutils.command.bdist_msi of Python 3.8
 """
+# Copyright (C) 2005, 2006 Martin von Löwis
+# Licensed to PSF under a Contributor Agreement.
+
+from __future__ import annotations
 
 import logging
-import msilib
 import os
 import shutil
 import sys
-from msilib import Dialog, Directory, Feature, add_data, schema, sequence, text
+from msilib import (  # pylint: disable=deprecated-module
+    CAB,
+    Binary,
+    Dialog,
+    Directory,
+    Feature,
+    Win64,
+    add_data,
+    add_tables,
+    gen_uuid,
+    init_database,
+    schema,
+    sequence,
+    text,
+)
 from sysconfig import get_platform, get_python_version
 
-from packaging.version import Version
 from setuptools import Command
-from setuptools.errors import OptionError
 
+from .._compat import packaging
+from ..exception import OptionError
 from ._pydialog import PyDialog
 
 
-# pylint: disable=attribute-defined-outside-init,missing-function-docstring
-# pylint: disable=too-many-lines,invalid-name
+# pylint: disable=invalid-name
 class bdist_msi(Command):
     """Create a Microsoft Installer (.msi) binary distribution."""
 
@@ -203,11 +216,11 @@ class bdist_msi(Command):
             os.unlink(installer_name)
 
         metadata = self.distribution.metadata
-        author = metadata.get_contact()
+        author = metadata.get_contact() or "UNKNOWN"
         version = metadata.get_version()
         # ProductVersion must be strictly numeric
         # XXX need to deal with prerelease versions
-        base_version = Version(version).base_version
+        base_version = packaging.version.Version(version).base_version
         # Prefix ProductName with Python x.y, so that
         # it sorts together with the other Python packages
         # in Add-Remove-Programs (APR)
@@ -216,15 +229,15 @@ class bdist_msi(Command):
             product_name = f"Python {self.target_version} {fullname}"
         else:
             product_name = f"Python {fullname}"
-        self.db = msilib.init_database(
+        self.db = init_database(
             installer_name,
             schema,
             product_name,
-            msilib.gen_uuid(),
+            gen_uuid(),
             base_version,
             author,
         )
-        msilib.add_tables(self.db, sequence)
+        add_tables(self.db, sequence)
         props = [("DistVersion", version)]
         email = metadata.author_email or metadata.maintainer_email
         if email:
@@ -255,7 +268,7 @@ class bdist_msi(Command):
 
     def add_files(self):
         db = self.db
-        cab = msilib.CAB("distfiles")
+        cab = CAB("distfiles")
         rootdir = os.path.abspath(self.bdist_dir)
 
         root = Directory(db, cab, None, rootdir, "TARGETDIR", "SourceDir")
@@ -264,7 +277,7 @@ class bdist_msi(Command):
         )
 
         items = [(feat, root, "")]
-        for version in self.versions + [self.other_version]:
+        for version in [*self.versions, self.other_version]:
             target = "TARGETDIR" + version
             name = default = "Python" + version
             desc = "Everything"
@@ -280,8 +293,8 @@ class bdist_msi(Command):
         db.Commit()
 
         seen = {}
-        for feature, directory, version in items:
-            todo = [directory]
+        for feature, base_directory, version in items:
+            todo = [base_directory]
             while todo:
                 directory = todo.pop()
                 for file in os.listdir(directory.absolute):
@@ -325,7 +338,7 @@ class bdist_msi(Command):
         cab.commit(db)
 
     def add_find_python(self):
-        """Adds code to the installer to compute the location of Python.
+        r"""Adds code to the installer to compute the location of Python.
 
         Properties PYTHON.MACHINE.X.Y and PYTHON.USER.X.Y will be set from the
         registry for each version of Python.
@@ -333,8 +346,8 @@ class bdist_msi(Command):
         Properties TARGETDIRX.Y will be set from PYTHON.USER.X.Y if defined,
         else from PYTHON.MACHINE.X.Y.
 
-        Properties PYTHONX.Y will be set to TARGETDIRX.Y\\python.exe"""
-
+        Properties PYTHONX.Y will be set to TARGETDIRX.Y\python.exe
+        """
         start = 402
         for ver in self.versions:
             install_path = rf"SOFTWARE\Python\PythonCore\{ver}\InstallPath"
@@ -347,11 +360,8 @@ class bdist_msi(Command):
             exe_action = f"PythonExe{ver}"
             target_dir_prop = f"TARGETDIR{ver}"
             exe_prop = f"PYTHON{ver}"
-            if msilib.Win64:
-                # type_: msidbLocatorTypeRawValue + msidbLocatorType64bit
-                type_ = 2 + 16
-            else:
-                type_ = 2
+            # type_: msidbLocatorTypeRawValue [ + msidbLocatorType64bit ]
+            type_ = 2 + 16 if Win64 else 2
             add_data(
                 self.db,
                 "RegLocator",
@@ -418,7 +428,7 @@ class bdist_msi(Command):
     def add_scripts(self):
         if self.install_script:
             start = 6800
-            for ver in self.versions + [self.other_version]:
+            for ver in [*self.versions, self.other_version]:
                 install_action = f"install_script.{ver}"
                 exe_prop = f"PYTHON{ver}"
                 add_data(
@@ -449,9 +459,7 @@ class bdist_msi(Command):
                 file.write('rem ="""\n%1 %0\nexit\n"""\n')
                 with open(self.pre_install_script, encoding="utf-8") as fin:
                     file.write(fin.read())
-            add_data(
-                self.db, "Binary", [("PreInstall", msilib.Binary(scriptfn))]
-            )
+            add_data(self.db, "Binary", [("PreInstall", Binary(scriptfn))])
             add_data(
                 self.db,
                 "CustomAction",
@@ -861,7 +869,7 @@ class bdist_msi(Command):
         c = seldlg.nextbutton("Next >", "Cancel")
         order = 1
         c.event("[TARGETDIR]", "[SourceDir]", ordering=order)
-        for version in self.versions + [self.other_version]:
+        for version in [*self.versions, self.other_version]:
             order += 1
             c.event(
                 "[TARGETDIR]",
@@ -1179,5 +1187,4 @@ class bdist_msi(Command):
             )
         else:
             base_name = f"{fullname}.{self.plat_name}.msi"
-        installer_name = os.path.join(self.dist_dir, base_name)
-        return installer_name
+        return os.path.join(self.dist_dir, base_name)
