@@ -1,14 +1,17 @@
 """Module Finder - discovers what modules are required by the code."""
 
+from __future__ import annotations
+
 import importlib.machinery
 import logging
+import os
 import sys
+from collections.abc import Sequence
 from contextlib import suppress
 from importlib.abc import ExecutionLoader
 from pathlib import Path, PurePath
-from sysconfig import get_config_var
 from types import CodeType
-from typing import Any, Dict, List, Optional, Sequence, Set, Tuple, Union
+from typing import Any, List, Tuple
 
 import opcode
 
@@ -24,7 +27,6 @@ from .module import ConstantsModule, Module
 
 BUILD_LIST = opcode.opmap["BUILD_LIST"]
 EXTENDED_ARG = opcode.opmap["EXTENDED_ARG"]
-INPLACE_ADD = opcode.opmap["INPLACE_ADD"]
 LOAD_CONST = opcode.opmap["LOAD_CONST"]
 IMPORT_NAME = opcode.opmap["IMPORT_NAME"]
 IMPORT_FROM = opcode.opmap["IMPORT_FROM"]
@@ -35,7 +37,6 @@ STORE_GLOBAL = opcode.opmap["STORE_GLOBAL"]
 STORE_OPS = (STORE_NAME, STORE_GLOBAL)
 HAVE_ARGUMENT = opcode.HAVE_ARGUMENT
 
-
 DeferredList = List[Tuple[Module, Module, List[str]]]
 
 __all__ = ["Module", "ModuleFinder"]
@@ -46,77 +47,52 @@ class ModuleFinder:
 
     def __init__(
         self,
-        include_files: Optional[IncludesList] = None,
-        excludes: Optional[List[str]] = None,
-        path: Optional[List[str]] = None,
-        replace_paths: Optional[List[Tuple[str, str]]] = None,
+        constants_module: ConstantsModule | None = None,
+        excludes: list[str] | None = None,
+        include_files: IncludesList | None = None,
+        path: list[str | Path] | None = None,
+        replace_paths: list[tuple[str, str]] | None = None,
+        zip_exclude_packages: Sequence[str] | None = None,
+        zip_include_packages: Sequence[str] | None = None,
         zip_include_all_packages: bool = False,
-        zip_exclude_packages: Optional[List[str]] = None,
-        zip_include_packages: Optional[List[str]] = None,
-        constants_module: Optional[ConstantsModule] = None,
-        zip_includes: Optional[IncludesList] = None,
+        zip_includes: IncludesList | None = None,
     ):
         self.included_files: InternalIncludesList = process_path_specs(
             include_files
         )
-        self.excludes: Dict[str, Any] = dict.fromkeys(excludes or [])
-        self.optimize_flag = 0
-        self.path: List[str] = self._set_path(path)
+        self.excludes: dict[str, Any] = dict.fromkeys(excludes or [])
+        self.optimize = 0
+        self.path: list[str] = list(map(os.fspath, path or sys.path))
         self.replace_paths = replace_paths or []
         self.zip_include_all_packages = zip_include_all_packages
-        self.zip_exclude_packages = zip_exclude_packages or []
-        self.zip_include_packages = zip_include_packages or []
+        self.zip_exclude_packages: set = zip_exclude_packages or set()
+        self.zip_include_packages: set = zip_include_packages or set()
         self.constants_module = constants_module
         self.zip_includes: InternalIncludesList = process_path_specs(
             zip_includes
         )
         self.modules = []
         self.aliases = {}
-        self.excluded_dependent_files: Set[Path] = set()
-        self._modules: Dict[str, Optional[Module]] = dict.fromkeys(
-            excludes or []
-        )
+        self.excluded_dependent_files: set[Path] = set()
+        self._modules: dict[str, Module | None] = dict.fromkeys(excludes or [])
         self._bad_modules = {}
-        self._hooks = __import__("cx_Freeze", fromlist=["hooks"]).hooks
-        self._hooks.initialize(self)
-        self._add_base_modules()
-
-    def _add_base_modules(self) -> None:
-        """
-        Add the base modules to the finder. These are the modules that
-        Python imports itself during initialization and, if not found,
-        can result in behavior that differs from running from source;
-        also include modules used within the bootstrap code.
-
-        When cx_Freeze is built, these modules (and modules they load) are
-        included in the startup zip file.
-        """
-        self.include_module("traceback")
-        self.include_module("warnings")
-        self.include_module("unicodedata")
-        self.include_package("encodings")
-        self.include_module("io")
-        self.include_module("os")
-        self.include_module("sys")
-        self.include_module("zlib")
-        self.include_module("collections.abc")
-        self.include_module("importlib.abc")
+        for name in self._base_hooks.exclude.MODULES:
+            self.exclude_module(name)
 
     def _add_module(
         self,
         name: str,
-        path: Optional[Sequence[Union[Path, str]]] = None,
-        file_name: Optional[Union[Path, str]] = None,
-        parent: Optional[Module] = None,
+        path: Sequence[Path | str] | None = None,
+        filename: Path | None = None,
+        parent: Module | None = None,
     ) -> Module:
-        """
-        Add a module to the list of modules but if one is already found,
+        """Add a module to the list of modules but if one is already found,
         then return it instead; this is done so that packages can be
         handled properly.
         """
         module = self._modules.get(name)
         if module is None:
-            module = Module(name, path, file_name, parent)
+            module = Module(name, path, filename, parent)
             self._modules[name] = module
             self.modules.append(module)
             if name in self._bad_modules:
@@ -131,12 +107,12 @@ class ModuleFinder:
             ):
                 module.in_file_system = 0
         if module.path is None and path is not None:
-            module.path = [Path(p) for p in path]
-        if module.file is None and file_name is not None:
-            module.file = file_name
+            module.path = list(map(Path, path))
+        if module.file is None and filename is not None:
+            module.file = filename
         return module
 
-    def _determine_parent(self, caller: Optional[Module]) -> Optional[Module]:
+    def _determine_parent(self, caller: Module | None) -> Module | None:
         """Determine the parent to use when searching packages."""
         if caller is not None:
             if caller.path is not None:
@@ -145,9 +121,24 @@ class ModuleFinder:
         return None
 
     @cached_property
-    def _builtin_modules(self) -> Set[str]:
+    def _base_hooks(self):
+        """Load the hooks dynamically to avoid cyclic errors, because hooks
+        have references to finder.
+        """
+        hooks = get_resource_file_path("hooks", "__init__", ".py")
+        fromlist = []
+        # add python files (modules)
+        for path in hooks.parent.glob("*.py"):
+            fromlist.append("hooks" if path.stem == "__init__" else path.stem)
+        # add directories (subpackages)
+        for path in hooks.parent.glob("*/__init__.py"):
+            fromlist.append(path.parent.stem)
+        return __import__("cx_Freeze.hooks", fromlist=fromlist)
+
+    @cached_property
+    def _builtin_modules(self) -> set[str]:
         """The built-in modules are determined based on the cx_Freeze build."""
-        builtin_modules: Set[str] = set(sys.builtin_module_names)
+        builtin_modules: set[str] = set(sys.builtin_module_names)
         dynload = get_resource_file_path("bases", "lib-dynload", "")
         if dynload and dynload.is_dir():
             # discard modules that exist in bases/lib-dynload
@@ -159,11 +150,10 @@ class ModuleFinder:
         self,
         caller: Module,
         package_module: Module,
-        from_list: List[str],
+        from_list: list[str],
         deferred_imports: DeferredList,
     ) -> None:
-        """
-        Ensure that the from list is satisfied. This is only necessary for
+        """Ensure that the from list is satisfied. This is only necessary for
         package modules. If the package module has not been completely
         imported yet, defer the import until it has been completely imported
         in order to avoid spurious errors about missing modules.
@@ -177,7 +167,7 @@ class ModuleFinder:
                 sub_module_name = f"{package_module.name}.{name}"
                 self._import_module(sub_module_name, deferred_imports, caller)
 
-    def _get_parent_by_name(self, name: str) -> Optional[Module]:
+    def _get_parent_by_name(self, name: str) -> Module | None:
         """Return the parent module given the name of a module."""
         pos = name.rfind(".")
         if pos > 0:
@@ -258,14 +248,12 @@ class ModuleFinder:
         self,
         name: str,
         deferred_imports: DeferredList,
-        caller: Optional[Module] = None,
+        caller: Module | None = None,
         relative_import_index: int = 0,
     ):
-        """
-        Attempt to find the named module and return it or None if no module
+        """Attempt to find the named module and return it or None if no module
         by that name could be found.
         """
-
         # absolute import (available in Python 2.5 and up)
         # the name given is the only name that will be searched
         if relative_import_index == 0:
@@ -273,7 +261,7 @@ class ModuleFinder:
 
         # old style relative import (regular 'import foo' in Python 2)
         # the name given is tried in the current package, and if no match
-        # is found, sys.path is searched for a top-level module/pockage
+        # is found, self.path is searched for a top-level module/pockage
         elif relative_import_index < 0:
             parent = self._determine_parent(caller)
             if parent is not None:
@@ -318,9 +306,8 @@ class ModuleFinder:
 
     def _internal_import_module(
         self, name: str, deferred_imports: DeferredList
-    ) -> Optional[Module]:
-        """
-        Internal method used for importing a module which assumes that the
+    ) -> Module | None:
+        """Internal method used for importing a module which assumes that the
         name given is an absolute name. None is returned if the module
         cannot be found.
         """
@@ -347,10 +334,7 @@ class ModuleFinder:
             if parent_module is None:
                 return None
             path = parent_module.path
-            if path is None:
-                path = self.path
-            else:
-                path = [str(p) for p in path]
+            path = self.path if path is None else list(map(os.fspath, path))
 
         if name in self.aliases:
             actual_name = self.aliases[name]
@@ -373,14 +357,14 @@ class ModuleFinder:
     def _load_module(
         self,
         name: str,
-        path: Optional[Sequence[str]],
+        path: Sequence[str] | None,
         deferred_imports: DeferredList,
-        parent: Optional[Module] = None,
-    ) -> Optional[Module]:
+        parent: Module | None = None,
+    ) -> Module | None:
         """Load the module, searching the module spec."""
-        spec: Optional[importlib.machinery.ModuleSpec] = None
-        loader: Optional[ExecutionLoader] = None
-        module: Optional[Module] = None
+        spec: importlib.machinery.ModuleSpec | None = None
+        loader: ExecutionLoader | None = None
+        module: Module | None = None
 
         # Find modules to load
         try:
@@ -398,7 +382,7 @@ class ModuleFinder:
                     parent=parent,
                 )
                 logging.debug("Adding module [%s] [PACKAGE]", name)
-                module.file = Path(path[0], "__init__.py")
+                module.file = Path(path[0]) / "__init__.py"
                 module.source_is_string = True
 
         if spec:
@@ -421,10 +405,10 @@ class ModuleFinder:
                     module.source_is_string = True
                 else:
                     logging.debug("Adding module [%s] [PACKAGE]", name)
-                    module.file = spec.origin  # path of __init__.py
+                    module.file = Path(spec.origin)  # path of __init__.py
             else:
                 module = self._add_module(
-                    name, file_name=spec.origin, parent=parent
+                    name, filename=Path(spec.origin), parent=parent
                 )
 
         if module is not None:
@@ -434,11 +418,11 @@ class ModuleFinder:
     def _load_module_code(
         self,
         module: Module,
-        loader: Optional[ExecutionLoader],
+        loader: ExecutionLoader | None,
         deferred_imports: DeferredList,
-    ) -> Optional[Module]:
+    ) -> Module | None:
         name = module.name
-        path = str(module.file)
+        path = os.fspath(module.file)
 
         if isinstance(loader, importlib.machinery.SourceFileLoader):
             logging.debug("Adding module [%s] [SOURCE]", name)
@@ -446,7 +430,7 @@ class ModuleFinder:
             source_bytes = loader.get_data(path)
             try:
                 module.code = loader.source_to_code(
-                    source_bytes, path, _optimize=self.optimize_flag
+                    source_bytes, path, _optimize=self.optimize
                 )
             except SyntaxError:
                 logging.debug("Invalid syntax in [%s]", name)
@@ -484,37 +468,36 @@ class ModuleFinder:
 
     def _load_module_from_file(
         self, name: str, filename: Path, deferred_imports: DeferredList
-    ) -> Optional[Module]:
+    ) -> Module | None:
         """Load the module from the filename."""
-        loader: Optional[ExecutionLoader] = None
+        loader: ExecutionLoader | None = None
 
         ext = filename.suffix
-        path = str(filename)
-        if ext == "" or ext in importlib.machinery.SOURCE_SUFFIXES:
+        path = os.fspath(filename)
+        if not ext or ext in importlib.machinery.SOURCE_SUFFIXES:
             loader = importlib.machinery.SourceFileLoader(name, path)
         elif ext in importlib.machinery.BYTECODE_SUFFIXES:
             loader = importlib.machinery.SourcelessFileLoader(name, path)
         elif ext in importlib.machinery.EXTENSION_SUFFIXES:
             loader = importlib.machinery.ExtensionFileLoader(name, path)
 
-        module = self._add_module(name, file_name=filename)
+        module = self._add_module(name, filename=filename)
         self._load_module_code(module, loader, deferred_imports)
         return module
 
     @staticmethod
     def _replace_package_in_code(module: Module) -> CodeType:
-        """
-        Replace the value of __package__ directly in the code,
+        """Replace the value of __package__ directly in the code,
         when the module is in a package and will be stored in library.zip.
         """
         code = module.code
         # Check if module is in a package and will be stored in library.zip
         # and is not defined in the module, like 'six' do
         if (
-            module.parent is None
-            or module.in_file_system >= 1
+            code is None
+            or module.parent is None
             or "__package__" in module.global_names
-            or code is None
+            or module.in_file_system >= 1
         ):
             return code
         # Only if the code references it.
@@ -537,10 +520,9 @@ class ModuleFinder:
         return code
 
     def _replace_paths_in_code(
-        self, module: Module, code: Optional[CodeType] = None
+        self, module: Module, code: CodeType | None = None
     ) -> CodeType:
-        """
-        Replace paths in the code as directed, returning a new code object
+        """Replace paths in the code as directed, returning a new code object
         with the modified paths in place.
         """
         top_level_module = module  # type: Module
@@ -576,37 +558,28 @@ class ModuleFinder:
                 )
 
         return code_object_replace(
-            code, co_consts=consts, co_filename=str(new_filename)
+            code, co_consts=consts, co_filename=os.fspath(new_filename)
         )
 
     def _run_hook(self, hook: str, module_name: str, *args) -> None:
-        """Run hook (load or missing) for the given module if one is
-        present."""
+        """Run hook (load or missing) for the given module if one is present.
+        For functions present in hooks.__init__:
+        package aiofiles -> load_aiofiles function
+        For functions in a separated module (lowercased):
+        package PyQt5, module QtCore -> pyqt5.load_pyqt5_qtcore.
+        """
+        base_hooks = self._base_hooks
         normalized_name = module_name.replace(".", "_")
-        method = getattr(self._hooks, f"{hook}_{normalized_name}", None)
+        method = getattr(base_hooks, f"{hook}_{normalized_name}", None)
+        if method is None:
+            root = module_name.split(".")[0].lower()
+            base_hooks = getattr(base_hooks, root, None)
+            if base_hooks is not None:
+                normalized_name = normalized_name.lower()
+                method = getattr(base_hooks, f"{hook}_{normalized_name}", None)
         if method is not None:
             method(self, *args)
 
-    @staticmethod
-    def _set_path(path: Optional[List[str]] = None) -> Set[str]:
-        """Set the path to search for modules, and fix the path for built-in
-        modules when it differs from the running python built-in modules."""
-        path = path or sys.path
-        if path == sys.path:
-            dynload = get_resource_file_path("bases", "lib-dynload", "")
-            if dynload and dynload.is_dir():
-                # add bases/lib-dynload to the finder path
-                insert_at = 0
-                dest_shared = get_config_var("DESTSHARED")
-                if dest_shared:
-                    for i, pth in enumerate(path):
-                        if pth == dest_shared:
-                            insert_at = i
-                            break
-                path = path.copy()
-                path.insert(insert_at, str(dynload))
-        return path
-
     def _scan_code(
         self,
         code,
@@ -616,7 +589,8 @@ class ModuleFinder:
     ):
         """Scan code, looking for imported modules and keeping track of the
         constants that have been created in order to better tell which
-        modules are truly missing."""
+        modules are truly missing.
+        """
         arguments = []
         imported_module = None
         co_code = code.co_code
@@ -640,7 +614,6 @@ class ModuleFinder:
             if opc == IMPORT_NAME:
                 name = code.co_names[arg]
                 if len(arguments) >= 2:
-                    # pylint: disable-next=W0632
                     relative_import_index, from_list = arguments[-2:]
                 else:
                     relative_import_index = -1
@@ -649,18 +622,17 @@ class ModuleFinder:
                     imported_module = self._import_module(
                         name, deferred_imports, module, relative_import_index
                     )
-                    if imported_module is not None:
-                        if (
-                            from_list
-                            and from_list != ("*",)
-                            and imported_module.path is not None
-                        ):
-                            self._ensure_from_list(
-                                module,
-                                imported_module,
-                                from_list,
-                                deferred_imports,
-                            )
+                    if imported_module is not None and (
+                        from_list
+                        and from_list != ("*",)
+                        and imported_module.path is not None
+                    ):
+                        self._ensure_from_list(
+                            module,
+                            imported_module,
+                            from_list,
+                            deferred_imports,
+                        )
 
             # import * statement: copy all global names
             elif (
@@ -692,21 +664,44 @@ class ModuleFinder:
         """
         self.aliases[name] = alias_for
 
+    def add_base_modules(self) -> None:
+        """Add the base modules to the finder. These are the modules that
+        Python imports itself during initialization and, if not found,
+        can result in behavior that differs from running from source;
+        also include modules used within the bootstrap code.
+
+        When cx_Freeze is built, these modules (and modules they load) are
+        included in the startup zip file.
+        """
+        self.include_package("collections")
+        self.include_package("encodings")
+        self.include_package("importlib")
+        self.include_module("io")
+        self.include_module("os")
+        self.include_module("sys")
+        self.include_module("traceback")
+        self.include_module("unicodedata")
+        self.include_module("warnings")
+        self.include_module("zlib")
+
     def add_constant(self, name: str, value: str) -> None:
         """Makes available a constant in the module BUILD_CONSTANTS which is
-        used in the initscripts."""
+        used in the initscripts.
+        """
         self.constants_module.values[name] = value
 
-    def exclude_dependent_files(self, filename: Union[Path, str]) -> None:
+    def exclude_dependent_files(self, filename: Path | str) -> None:
         """Exclude the dependent files of the named file from the resulting
-        frozen executable."""
+        frozen executable.
+        """
         if not isinstance(filename, Path):
             filename = Path(filename)
         self.excluded_dependent_files.add(filename)
 
     def exclude_module(self, name: str) -> None:
         """Exclude the named module and its submodules from the resulting
-        frozen executable."""
+        frozen executable.
+        """
         modules_to_exclude = [name] + [
             mod for mod in self._modules if mod.startswith(f"{name}.")
         ]
@@ -715,7 +710,7 @@ class ModuleFinder:
             self._modules[mod] = None
 
     def include_file_as_module(
-        self, path: Union[Path, str], name: Optional[str] = None
+        self, path: Path | str, name: str | None = None
     ) -> Module:
         """Include the named file as a module in the frozen executable."""
         if isinstance(path, str):
@@ -724,13 +719,18 @@ class ModuleFinder:
             name = path.name.partition(".")[0]
         deferred_imports: DeferredList = []
         module = self._load_module_from_file(name, path, deferred_imports)
+        if module is not None:
+            parent = self._get_parent_by_name(name)
+            if parent is not None:
+                parent.global_names.add(module.name)
+                module.parent = parent
         self._import_deferred_imports(deferred_imports)
         return module
 
     def include_files(
         self,
-        source_path: Union[Path, str],
-        target_path: Union[Path, str],
+        source_path: Path | str,
+        target_path: Path | str,
         copy_dependent_files: bool = True,
     ) -> None:
         """Include the files in the given directory in the target build."""
@@ -743,10 +743,9 @@ class ModuleFinder:
     def include_module(self, name: str) -> Module:
         """Include the named module in the frozen executable."""
         # includes has priority over excludes
-        if name in self.excludes and name in self._modules:
-            if self._modules[name] is None:
-                self.excludes.pop(name)
-                self._modules.pop(name)
+        if name in self.excludes and self._modules.get(name) is None:
+            self.excludes.pop(name)
+            self._modules.pop(name, None)
         # include module
         deferred_imports: DeferredList = []
         module = self._import_module(name, deferred_imports)
@@ -755,7 +754,8 @@ class ModuleFinder:
 
     def include_package(self, name: str) -> Module:
         """Include the named package and any submodules in the frozen
-        executable."""
+        executable.
+        """
         deferred_imports: DeferredList = []
         module = self._import_module(name, deferred_imports)
         if module.path:
@@ -777,23 +777,24 @@ class ModuleFinder:
             print("may not be needed on this platform.\n")
 
     @property
-    def optimize_flag(self) -> int:
+    def optimize(self) -> int:
         """The value of optimize flag propagated according to the user's
-        choice."""
+        choice.
+        """
         return self._optimize_flag
 
-    @optimize_flag.setter
-    def optimize_flag(self, value: int):
-        # The value of optimize_flag is checked in dist.py or main,py. This
-        # value is unlikely to be wrong, yet we check and ignore any divergent
-        # value.
+    @optimize.setter
+    def optimize(self, value: int):
+        # The value of optimize is checked in '.command.build_exe' or '.cli'.
+        # This value is unlikely to be wrong, yet we check and ignore any
+        # divergent value.
         if -1 <= value <= 2:
             self._optimize_flag = value
 
     def zip_include_files(
         self,
-        source_path: Union[str, Path],
-        target_path: Optional[Union[str, Path, PurePath]] = None,
+        source_path: str | Path,
+        target_path: str | Path | PurePath | None = None,
     ) -> None:
         """Include files or all of the files in a directory to the zip file."""
         self.zip_includes.extend(
