@@ -1,21 +1,20 @@
 """A collection of functions which are triggered automatically by finder when
-certain packages are included or not found."""
-# pylint: disable=unused-argument,invalid-name
+certain packages are included or not found.
+"""
+# pylint: disable=invalid-name
+# ruff: noqa: ARG001
+
+from __future__ import annotations
 
-import os
 import sys
 import sysconfig
+from contextlib import suppress
 from pathlib import Path
-from typing import Optional
 
-from ..common import code_object_replace, get_resource_file_path
+from .._compat import IS_MINGW, IS_WINDOWS
 from ..finder import ModuleFinder
 from ..module import Module
-from ._qthooks import get_qt_plugins_paths  # noqa
-
-DARWIN = sys.platform == "darwin"
-MINGW = sysconfig.get_platform().startswith("mingw")
-WIN32 = sys.platform == "win32"
+from ._qthooks import get_qt_plugins_paths  # noqa: F401
 
 
 def load_aiofiles(finder: ModuleFinder, module: Module) -> None:
@@ -36,41 +35,51 @@ def load_babel(finder: ModuleFinder, module: Module) -> None:
 
 def load_bcrypt(finder: ModuleFinder, module: Module) -> None:
     """The bcrypt < 4.0 package requires the _cffi_backend module
-    (loaded implicitly)."""
+    (loaded implicitly).
+    """
     include_cffi = True
-    if module.distribution:
-        if int(module.distribution.version.split(".")[0]) >= 4:
-            include_cffi = False
+    if (
+        module.distribution
+        and int(module.distribution.version.split(".")[0]) >= 4
+    ):
+        include_cffi = False
     if include_cffi:
         finder.include_module("_cffi_backend")
 
 
 def load_boto(finder: ModuleFinder, module: Module) -> None:
-    """the boto package uses 'six' fake modules."""
+    """The boto package uses 'six' fake modules."""
     finder.exclude_module("boto.vendored.six.moves")
 
 
+def load_boto3(finder: ModuleFinder, module: Module) -> None:
+    """The boto3 package."""
+    finder.include_package("boto3.dynamodb")
+    finder.include_package("boto3.ec2")
+    finder.include_package("boto3.s3")
+    finder.include_files(module.file.parent / "data", "lib/boto3/data")
+
+
 def load_cElementTree(finder: ModuleFinder, module: Module) -> None:
     """The cElementTree module implicitly loads the elementtree.ElementTree
-    module; make sure this happens."""
+    module; make sure this happens.
+    """
     finder.include_module("elementtree.ElementTree")
 
 
 def load_ceODBC(finder: ModuleFinder, module: Module) -> None:
     """The ceODBC module implicitly imports both datetime and decimal;
-    make sure this happens."""
+    make sure this happens.
+    """
     finder.include_module("datetime")
     finder.include_module("decimal")
 
 
 def load_certifi(finder: ModuleFinder, module: Module) -> None:
-    """The certifi package, in python 3.7 and up, uses importlib.resources
-    to locate the cacert.pem in zip packages.
-    In previous versions, it is expected to be stored in the file system."""
+    """The certifi package uses importlib.resources to locate the cacert.pem
+    in zip packages.
+    """
     if module.in_file_system == 0:
-        if sys.version_info < (3, 7):
-            module.in_file_system = 1
-            return
         cacert = Path(__import__("certifi").where())
         finder.zip_include_files(cacert, Path("certifi", cacert.name))
 
@@ -84,29 +93,29 @@ def load_cffi_cparser(finder: ModuleFinder, module: Module) -> None:
     """The cffi.cparser module can use a extension if present."""
     try:
         cffi = __import__("cffi", fromlist=["_pycparser"])
-        pycparser = getattr(cffi, "_pycparser")
+        pycparser = getattr(cffi, "_pycparser")  # noqa: B009
         finder.include_module(pycparser.__name__)
     except (ImportError, AttributeError):
         finder.exclude_module("cffi._pycparser")
 
 
+def load_charset_normalizer(finder: ModuleFinder, module: Module) -> None:
+    """The charset_normalizer package."""
+    finder.exclude_module("charset_normalizer.cli")
+
+
+def load_charset_normalizer_md(finder: ModuleFinder, module: Module) -> None:
+    """The charset_normalizer package implicitly imports a extension module."""
+    mypyc = module.file.parent / ("md__mypyc" + "".join(module.file.suffixes))
+    if mypyc.exists():
+        finder.include_module("charset_normalizer.md__mypyc")
+
+
 def load_crc32c(finder: ModuleFinder, module: Module) -> None:
     """The google.crc32c module requires _cffi_backend module."""
     finder.include_module("_cffi_backend")
 
 
-def load_clr(finder: ModuleFinder, module: Module) -> None:
-    """The pythonnet package (imported as 'clr') needs Python.Runtime.dll
-    in runtime."""
-    dll_name = "Python.Runtime.dll"
-    dll_path = module.file.parent / dll_name
-    if not dll_path.exists():
-        dll_path = module.file.parent / "pythonnet/runtime" / dll_name
-        if not dll_path.exists():
-            return
-    finder.include_files(dll_path, Path("lib", dll_name))
-
-
 def load_cryptography(finder: ModuleFinder, module: Module) -> None:
     """The cryptography module requires the _cffi_backend module."""
     if module.distribution and module.distribution.requires:
@@ -123,8 +132,9 @@ def load_cryptography(finder: ModuleFinder, module: Module) -> None:
 
 def load__ctypes(finder: ModuleFinder, module: Module) -> None:
     """In Windows, the _ctypes module in Python 3.8+ requires an additional
-    libffi dll to be present in the build directory."""
-    if WIN32 and sys.version_info >= (3, 8) and not MINGW:
+    libffi dll to be present in the build directory.
+    """
+    if IS_WINDOWS and sys.version_info >= (3, 8):
         dll_pattern = "libffi-*.dll"
         dll_dir = Path(sys.base_prefix, "DLLs")
         for dll_path in dll_dir.glob(dll_pattern):
@@ -133,7 +143,8 @@ def load__ctypes(finder: ModuleFinder, module: Module) -> None:
 
 def load_cx_Oracle(finder: ModuleFinder, module: Module) -> None:
     """The cx_Oracle module implicitly imports datetime; make sure this
-    happens."""
+    happens.
+    """
     finder.include_module("datetime")
     finder.include_module("decimal")
 
@@ -143,15 +154,22 @@ def load_datetime(finder: ModuleFinder, module: Module) -> None:
     finder.include_module("time")
 
 
+def load_discord(finder: ModuleFinder, module: Module) -> None:
+    """py-cord requires its metadata."""
+    module.update_distribution("py-cord")
+
+
 def load_docutils_frontend(finder: ModuleFinder, module: Module) -> None:
     """The optik module is the old name for the optparse module; ignore the
-    module if it cannot be found."""
+    module if it cannot be found.
+    """
     module.ignore_names.add("optik")
 
 
 def load_dummy_threading(finder: ModuleFinder, module: Module) -> None:
     """The dummy_threading module plays games with the name of the threading
-    module for its own purposes; ignore that here."""
+    module for its own purposes; ignore that here.
+    """
     finder.exclude_module("_dummy_threading")
 
 
@@ -162,7 +180,8 @@ def load_flask_compress(finder: ModuleFinder, module: Module) -> None:
 
 def load_ftplib(finder: ModuleFinder, module: Module) -> None:
     """The ftplib module attempts to import the SOCKS module; ignore this
-    module if it cannot be found."""
+    module if it cannot be found.
+    """
     module.ignore_names.add("SOCKS")
 
 
@@ -172,90 +191,10 @@ def load_gevent(finder: ModuleFinder, module: Module) -> None:
 
 
 def load_GifImagePlugin(finder: ModuleFinder, module: Module) -> None:
-    """The GifImagePlugin module optionally imports the _imaging_gif module"""
+    """The GifImagePlugin module optionally imports the _imaging_gif module."""
     module.ignore_names.add("_imaging_gif")
 
 
-def load_glib(finder: ModuleFinder, module: Module) -> None:
-    """Ignore globals that are imported."""
-    module.global_names.update(
-        [
-            "GError",
-            "IOChannel",
-            "IO_ERR",
-            "IO_FLAG_APPEND",
-            "IO_FLAG_GET_MASK",
-            "IO_FLAG_IS_READABLE",
-            "IO_FLAG_IS_SEEKABLE",
-            "IO_FLAG_IS_WRITEABLE",
-            "IO_FLAG_MASK",
-            "IO_FLAG_NONBLOCK",
-            "IO_FLAG_SET_MASK",
-            "IO_HUP",
-            "IO_IN",
-            "IO_NVAL",
-            "IO_OUT",
-            "IO_PRI",
-            "IO_STATUS_AGAIN",
-            "IO_STATUS_EOF",
-            "IO_STATUS_ERROR",
-            "IO_STATUS_NORMAL",
-            "Idle",
-            "MainContext",
-            "MainLoop",
-            "OPTION_ERROR",
-            "OPTION_ERROR_BAD_VALUE",
-            "OPTION_ERROR_FAILED",
-            "OPTION_ERROR_UNKNOWN_OPTION",
-            "OPTION_FLAG_FILENAME",
-            "OPTION_FLAG_HIDDEN",
-            "OPTION_FLAG_IN_MAIN",
-            "OPTION_FLAG_NOALIAS",
-            "OPTION_FLAG_NO_ARG",
-            "OPTION_FLAG_OPTIONAL_ARG",
-            "OPTION_FLAG_REVERSE",
-            "OPTION_REMAINING",
-            "OptionContext",
-            "OptionGroup",
-            "PRIORITY_DEFAULT",
-            "PRIORITY_DEFAULT_IDLE",
-            "PRIORITY_HIGH",
-            "PRIORITY_HIGH_IDLE",
-            "PRIORITY_LOW",
-            "Pid",
-            "PollFD",
-            "SPAWN_CHILD_INHERITS_STDIN",
-            "SPAWN_DO_NOT_REAP_CHILD",
-            "SPAWN_FILE_AND_ARGV_ZERO",
-            "SPAWN_LEAVE_DESCRIPTORS_OPEN",
-            "SPAWN_SEARCH_PATH",
-            "SPAWN_STDERR_TO_DEV_NULL",
-            "SPAWN_STDOUT_TO_DEV_NULL",
-            "Source",
-            "Timeout",
-            "child_watch_add",
-            "filename_display_basename",
-            "filename_display_name",
-            "filename_from_utf8",
-            "get_application_name",
-            "get_current_time",
-            "get_prgname",
-            "glib_version",
-            "idle_add",
-            "io_add_watch",
-            "main_context_default",
-            "main_depth",
-            "markup_escape_text",
-            "set_application_name",
-            "set_prgname",
-            "source_remove",
-            "spawn_async",
-            "timeout_add",
-            "timeout_add_seconds",
-        ]
-    )
-
-
 def load_googleapiclient(finder: ModuleFinder, module: Module) -> None:
     """Add the googleapiclient metadata for googleapiclient package."""
     module.update_distribution("google_api_python_client")
@@ -265,7 +204,8 @@ def load_googleapiclient_discovery(
     finder: ModuleFinder, module: Module
 ) -> None:
     """The googleapiclient.discovery module needs discovery_cache subpackage
-    in file system."""
+    in file system.
+    """
     discovery_cache = finder.include_package("googleapiclient.discovery_cache")
     discovery_cache.in_file_system = 1
 
@@ -285,7 +225,7 @@ def load_gtk__gtk(finder: ModuleFinder, module: Module) -> None:
 
 
 def load_h5py(finder: ModuleFinder, module: Module) -> None:
-    """h5py module has a number of implicit imports"""
+    """h5py module has a number of implicit imports."""
     finder.include_module("h5py.defs")
     finder.include_module("h5py.utils")
     finder.include_module("h5py._proxy")
@@ -299,19 +239,20 @@ def load_h5py(finder: ModuleFinder, module: Module) -> None:
 
 
 def load_h5py_wrapper(finder: ModuleFinder, module: Module) -> None:
-    """h5py_wrapper module requires future and pytest-runner"""
+    """h5py_wrapper module requires future and pytest-runner."""
     finder.include_module("future")
     finder.include_module("ptr")
 
 
 def load_hashlib(finder: ModuleFinder, module: Module) -> None:
-    """hashlib's fallback modules don't exist if the equivalent OpenSSL
-    algorithms are loaded from _hashlib, so we can ignore the error."""
+    """The hashlib's fallback modules don't exist if the equivalent OpenSSL
+    algorithms are loaded from _hashlib, so we can ignore the error.
+    """
     module.ignore_names.update(["_md5", "_sha", "_sha256", "_sha512"])
 
 
 def load_hdfdict(finder: ModuleFinder, module: Module) -> None:
-    """hdfdict module requires h5py_wrapper and PyYAML"""
+    """The hdfdict module requires h5py_wrapper and PyYAML."""
     finder.include_module("h5py_wrapper")
     finder.include_package("yaml")
 
@@ -321,9 +262,31 @@ def load_idna(finder: ModuleFinder, module: Module) -> None:
     finder.include_module("idna.idnadata")
 
 
-def load_lxml(finder: ModuleFinder, module: Module) -> None:
-    """The lxml package uses an extension."""
-    finder.include_module("lxml._elementpath")
+def load_imagej(finder: ModuleFinder, module: Module) -> None:
+    """The pyimagej package requires its metadata."""
+    module.update_distribution("pyimagej")
+
+
+def load_jpype(finder: ModuleFinder, module: Module) -> None:
+    """The JPype1 package requires its binary."""
+    source = module.file.parent.parent / "org.jpype.jar"
+    if source.exists():
+        finder.include_files(
+            source, f"lib/{source.name}", copy_dependent_files=False
+        )
+
+
+def load_lazy_loader(finder: ModuleFinder, module: Module) -> None:
+    """Use load_lazy_loader 0.2+ to work with .pyc files."""
+    if module.distribution.version < "0.2":
+        raise SystemExit(
+            "Please upgrade 'lazy_loader>=0.2' to support cx_Freeze"
+        )
+
+
+def load_librosa(finder: ModuleFinder, module: Module) -> None:
+    """The librosa must be loaded as package."""
+    finder.include_package("librosa")
 
 
 def load_llvmlite(finder: ModuleFinder, module: Module) -> None:
@@ -332,46 +295,22 @@ def load_llvmlite(finder: ModuleFinder, module: Module) -> None:
     finder.exclude_module("llvmlite.tests")
 
 
-def load_matplotlib(finder: ModuleFinder, module: Module) -> None:
-    """The matplotlib package requires mpl-data subdirectory."""
-    data_path = module.path[0] / "mpl-data"
-    target_path = Path("lib", module.name, "mpl-data")
-    # After matplotlib 3.4 mpl-data is guaranteed to be a subdirectory.
-    if not data_path.is_dir():
-        data_path = __import__("matplotlib").get_data_path()
-        need_patch = True
-    else:
-        need_patch = module.in_file_system == 0
-    finder.include_files(data_path, target_path, copy_dependent_files=False)
-    finder.include_package("matplotlib")
-    finder.exclude_module("matplotlib.tests")
-    finder.exclude_module("matplotlib.testing")
-    if not need_patch or module.code is None:
-        return
-    code_to_inject = f"""
-def _get_data_path():
-    import os, sys
-    return os.path.join(os.path.dirname(sys.executable), "{target_path!s}")
-"""
-    for code_str in [
-        code_to_inject,
-        code_to_inject.replace("_get_data_path", "get_data_path"),
-    ]:
-        new_code = compile(code_str, str(module.file), "exec")
-        co_func = new_code.co_consts[0]
-        name = co_func.co_name
-        code = module.code
-        consts = list(code.co_consts)
-        for i, constant in enumerate(consts):
-            if isinstance(constant, type(code)) and constant.co_name == name:
-                consts[i] = co_func
-                break
-        module.code = code_object_replace(code, co_consts=consts)
+def load_lxml(finder: ModuleFinder, module: Module) -> None:
+    """The lxml package uses an extension."""
+    finder.include_module("lxml._elementpath")
+
+
+def load_markdown(finder: ModuleFinder, module: Module) -> None:
+    """The markdown package implicitly loads html.parser; make sure this
+    happens.
+    """
+    finder.include_module("html.parser")
 
 
 def load_Numeric(finder: ModuleFinder, module: Module) -> None:
     """The Numeric module optionally loads the dotblas module; ignore the error
-    if this modules does not exist."""
+    if this modules does not exist.
+    """
     module.ignore_names.add("dotblas")
 
 
@@ -386,10 +325,32 @@ def load_orjson(finder: ModuleFinder, module: Module) -> None:
     finder.include_package("zoneinfo")
 
 
-def load_pandas(finder: ModuleFinder, module: Module) -> None:
-    """The pandas has dynamic imports."""
-    finder.include_package("pandas._libs")
-    finder.exclude_module("pandas.tests")
+def load_os(finder: ModuleFinder, module: Module) -> None:
+    """The os module should filter import names."""
+    if IS_WINDOWS:
+        module.exclude_names.add("posix")
+    else:
+        module.exclude_names.add("nt")
+
+
+def load_pathlib(finder: ModuleFinder, module: Module) -> None:
+    """The pathlib module should filter import names."""
+    if IS_WINDOWS:
+        module.exclude_names.update(("grp", "pwd"))
+    else:
+        module.exclude_names.add("nt")
+
+
+def load_pickle(finder: ModuleFinder, module: Module) -> None:
+    """The pickle module uses doctest for tests and shouldn't be imported."""
+    module.exclude_names.add("doctest")
+    if not sys.platform.startswith("java"):
+        module.exclude_names.add("org.python.core")
+
+
+def load_pickletools(finder: ModuleFinder, module: Module) -> None:
+    """The pickletools module uses doctest that shouldn't be imported."""
+    module.exclude_names.add("doctest")
 
 
 def load_pikepdf(finder: ModuleFinder, module: Module) -> None:
@@ -397,11 +358,6 @@ def load_pikepdf(finder: ModuleFinder, module: Module) -> None:
     finder.include_package("pikepdf")
 
 
-def load_PIL(finder: ModuleFinder, module: Module) -> None:
-    """The Pillow must be loaded as a package."""
-    finder.include_package("PIL")
-
-
 def load_plotly(finder: ModuleFinder, module: Module) -> None:
     """The plotly must be loaded as a package."""
     finder.include_package("plotly")
@@ -409,13 +365,15 @@ def load_plotly(finder: ModuleFinder, module: Module) -> None:
 
 def load_pkg_resources(finder: ModuleFinder, module: Module) -> None:
     """The pkg_resources must be loaded as a package;
-    dynamically loaded modules in subpackages is growing."""
+    dynamically loaded modules in subpackages is growing.
+    """
     finder.include_package("pkg_resources")
 
 
 def load_postgresql_lib(finder: ModuleFinder, module: Module) -> None:
     """The postgresql.lib module requires the libsys.sql file to be included
-    so make sure that file is included."""
+    so make sure that file is included.
+    """
     libsys = module.path[0] / "libsys.sql"
     if libsys.exists():
         finder.include_files(libsys, libsys.name)
@@ -427,7 +385,7 @@ def load_pty(finder: ModuleFinder, module: Module) -> None:
 
 
 def load_ptr(finder: ModuleFinder, module: Module) -> None:
-    """pytest-runner requires its metadata"""
+    """pytest-runner requires its metadata."""
     module.update_distribution("pytest-runner")
 
 
@@ -439,7 +397,8 @@ def load_pycountry(finder: ModuleFinder, module: Module) -> None:
 
 def load_pycparser(finder: ModuleFinder, module: Module) -> None:
     """These files are missing which causes
-    permission denied issues on windows when they are regenerated."""
+    permission denied issues on windows when they are regenerated.
+    """
     finder.include_module("pycparser.lextab")
     finder.include_module("pycparser.yacctab")
 
@@ -467,11 +426,18 @@ def load_pygments(finder: ModuleFinder, module: Module) -> None:
 
 def load_pyodbc(finder: ModuleFinder, module: Module) -> None:
     """The pyodbc module implicitly imports others modules;
-    make sure this happens."""
+    make sure this happens.
+    """
     for mod in ("datetime", "decimal", "hashlib", "locale", "uuid"):
         finder.include_module(mod)
 
 
+def load_pyreadstat(finder: ModuleFinder, module: Module) -> None:
+    """The pyreadstat package must be loaded as a package."""
+    finder.include_package("pyreadstat")
+    finder.include_module("pandas")
+
+
 def load_pyqtgraph(finder: ModuleFinder, module: Module) -> None:
     """The pyqtgraph package must be loaded as a package."""
     finder.include_package("pyqtgraph")
@@ -479,7 +445,8 @@ def load_pyqtgraph(finder: ModuleFinder, module: Module) -> None:
 
 def load_pytest(finder: ModuleFinder, module: Module) -> None:
     """The pytest package implicitly imports others modules;
-    make sure this happens."""
+    make sure this happens.
+    """
     pytest = __import__("pytest")
     for mod in pytest.freeze_includes():
         finder.include_module(mod)
@@ -490,7 +457,8 @@ def load_pythoncom(finder: ModuleFinder, module: Module) -> None:
     cannot be loaded directly in Python 2.5 and higher a special module is
     used to perform that task; simply use that technique directly to
     determine the name of the DLL and ensure it is included as a file in
-    the target directory."""
+    the target directory.
+    """
     pythoncom = __import__("pythoncom")
     filename = Path(pythoncom.__file__)
     finder.include_files(
@@ -498,37 +466,13 @@ def load_pythoncom(finder: ModuleFinder, module: Module) -> None:
     )
 
 
-def load_pytz(finder: ModuleFinder, module: Module) -> None:
-    """The pytz module requires timezone data to be found in a known directory
-    or in the zip file where the package is written."""
-    target_path = Path("lib", "pytz", "zoneinfo")
-    data_path = module.path[0] / "zoneinfo"
-    if not data_path.is_dir():
-        # Fedora (and possibly other systems) use a separate location to
-        # store timezone data so look for that here as well
-        pytz = __import__("pytz")
-        data_path = Path(
-            getattr(pytz, "_tzinfo_dir", None)
-            or os.getenv("PYTZ_TZDATADIR")
-            or "/usr/share/zoneinfo"
-        )
-        if data_path.is_dir():
-            finder.add_constant("PYTZ_TZDATADIR", str(target_path))
-    if data_path.is_dir():
-        if module.in_file_system >= 1:
-            finder.include_files(
-                data_path, target_path, copy_dependent_files=False
-            )
-        else:
-            finder.zip_include_files(data_path, Path("pytz", "zoneinfo"))
-
-
 def load_pywintypes(finder: ModuleFinder, module: Module) -> None:
     """The pywintypes module is actually contained in a DLL but since those
     cannot be loaded directly in Python 2.5 and higher a special module is
     used to perform that task; simply use that technique directly to
     determine the name of the DLL and ensure it is included as a file in the
-    target directory."""
+    target directory.
+    """
     pywintypes = __import__("pywintypes")
     filename = Path(pywintypes.__file__)
     finder.include_files(
@@ -538,12 +482,21 @@ def load_pywintypes(finder: ModuleFinder, module: Module) -> None:
 
 def load_reportlab(finder: ModuleFinder, module: Module) -> None:
     """The reportlab module loads a submodule rl_settings via exec so force
-    its inclusion here."""
+    its inclusion here.
+    """
     finder.include_module("reportlab.rl_settings")
 
 
-def load_sentry(finder: ModuleFinder, module: Module) -> None:
-    """The Sentry.io SDK"""
+def load_shapely(finder: ModuleFinder, module: Module) -> None:
+    """The Shapely.libs directory is not copied."""
+    libs_name = "Shapely.libs"
+    source_dir = module.path[0].parent / libs_name
+    if source_dir.exists():
+        finder.include_files(source_dir, f"lib/{libs_name}")
+
+
+def load_sentry_sdk(finder: ModuleFinder, module: Module) -> None:
+    """The Sentry.io SDK."""
     finder.include_module("sentry_sdk.integrations.stdlib")
     finder.include_module("sentry_sdk.integrations.excepthook")
     finder.include_module("sentry_sdk.integrations.dedupe")
@@ -554,61 +507,38 @@ def load_sentry(finder: ModuleFinder, module: Module) -> None:
     finder.include_module("sentry_sdk.integrations.threading")
 
 
-def load_skimage(finder: ModuleFinder, module: Module) -> None:
-    """The skimage package."""
-    finder.include_package("skimage.io")
-    # exclude all tests
-    finder.exclude_module("skimage.color.tests")
-    finder.exclude_module("skimage.data.tests")
-    finder.exclude_module("skimage.draw.tests")
-    finder.exclude_module("skimage.exposure.tests")
-    finder.exclude_module("skimage.feature.tests")
-    finder.exclude_module("skimage.filters.tests")
-    finder.exclude_module("skimage.graph.tests")
-    finder.exclude_module("skimage.io.tests")
-    finder.exclude_module("skimage.measure.tests")
-    finder.exclude_module("skimage.metrics.tests")
-    finder.exclude_module("skimage.morphology.tests")
-    finder.exclude_module("skimage.restoration.tests")
-    finder.exclude_module("skimage.segmentation.tests")
-    finder.exclude_module("skimage._shared.tests")
-    finder.exclude_module("skimage.transform.tests")
-    finder.exclude_module("skimage.util.tests")
-    finder.exclude_module("skimage.viewer.tests")
-
-
-def load_skimage_feature_orb_cy(finder: ModuleFinder, module: Module) -> None:
-    """The skimage.feature.orb_cy is a extension that load a module."""
-    finder.include_module("skimage.feature._orb_descriptor_positions")
-
-
 def load_setuptools(finder: ModuleFinder, module: Module) -> None:
     """The setuptools must be loaded as a package, to prevent it to break in
-    the future."""
+    the future.
+    """
     finder.include_package("setuptools")
 
 
 def load_setuptools_extension(finder: ModuleFinder, module: Module) -> None:
     """The setuptools.extension module optionally loads
-    Pyrex.Distutils.build_ext but its absence is not considered an error."""
+    Pyrex.Distutils.build_ext but its absence is not considered an error.
+    """
     module.ignore_names.add("Pyrex.Distutils.build_ext")
 
 
 def load_site(finder: ModuleFinder, module: Module) -> None:
     """The site module optionally loads the sitecustomize and usercustomize
-    modules; ignore the error if these modules do not exist."""
+    modules; ignore the error if these modules do not exist.
+    """
     module.ignore_names.update(["sitecustomize", "usercustomize"])
 
 
 def load_sqlite3(finder: ModuleFinder, module: Module) -> None:
     """In Windows, the sqlite3 module requires an additional dll sqlite3.dll to
-    be present in the build directory."""
-    if WIN32 and not MINGW:
+    be present in the build directory.
+    """
+    if IS_WINDOWS:
         dll_name = "sqlite3.dll"
         dll_path = Path(sys.base_prefix, "DLLs", dll_name)
         if not dll_path.exists():
             dll_path = Path(sys.base_prefix, "Library", "bin", dll_name)
-        finder.include_files(dll_path, Path("lib", dll_name))
+        if dll_path.exists():
+            finder.include_files(dll_path, Path("lib", dll_name))
     finder.include_package("sqlite3")
 
 
@@ -617,26 +547,15 @@ def load_six(finder: ModuleFinder, module: Module) -> None:
     finder.exclude_module("six.moves")
 
 
-def load_ssl(finder: ModuleFinder, module: Module) -> None:
-    """In Windows, the SSL module in Python 3.7+ requires additional dlls to
-    be present in the build directory."""
-    if WIN32 and sys.version_info >= (3, 7) and not MINGW:
-        for dll_search in ["libcrypto-*.dll", "libssl-*.dll"]:
-            libs_dir = Path(sys.base_prefix, "DLLs")
-            for dll_path in libs_dir.glob(dll_search):
-                finder.include_files(dll_path, Path("lib", dll_path.name))
-
-
 def load_sysconfig(finder: ModuleFinder, module: Module) -> None:
     """The sysconfig module implicitly loads _sysconfigdata."""
+    if IS_WINDOWS:
+        return
     get_data_name = getattr(sysconfig, "_get_sysconfigdata_name", None)
     if get_data_name is None:
-        datafile = "_sysconfigdata"
-    else:
-        if not hasattr(sys, "abiflags"):
-            sys.abiflags = ""
-        datafile = get_data_name()
-    finder.include_module(datafile)
+        return
+    with suppress(ImportError):
+        finder.include_module(get_data_name())
 
 
 def load_tensorflow(finder: ModuleFinder, module: Module) -> None:
@@ -651,39 +570,6 @@ def load_time(finder: ModuleFinder, module: Module) -> None:
     finder.include_module("_strptime")
 
 
-def load_tkinter(finder: ModuleFinder, module: Module) -> None:
-    """The tkinter module has data files (also called tcl/tk libraries) that
-    are required to be loaded at runtime."""
-    folders = []
-    tcltk = get_resource_file_path("bases", "tcltk", "")
-    if tcltk and tcltk.is_dir():
-        # manylinux wheels and macpython wheels store tcl/tk libraries
-        folders.append(("TCL_LIBRARY", list(tcltk.glob("tcl*"))[0]))
-        folders.append(("TK_LIBRARY", list(tcltk.glob("tk*"))[0]))
-    else:
-        # Windows, MSYS2, Miniconda: collect the tcl/tk libraries
-        try:
-            tkinter = __import__("tkinter")
-        except (ImportError, AttributeError):
-            return
-        root = tkinter.Tk(useTk=False)
-        source_path = Path(root.tk.exprstring("$tcl_library"))
-        folders.append(("TCL_LIBRARY", source_path))
-        source_name = source_path.name.replace("tcl", "tk")
-        source_path = source_path.parent / source_name
-        folders.append(("TK_LIBRARY", source_path))
-    for env_name, source_path in folders:
-        target_path = Path("lib", "tcltk", source_path.name)
-        finder.add_constant(env_name, str(target_path))
-        finder.include_files(source_path, target_path)
-        if WIN32 and not MINGW:
-            dll_name = source_path.name.replace(".", "") + "t.dll"
-            dll_path = Path(sys.base_prefix, "DLLs", dll_name)
-            if not dll_path.exists():
-                continue
-            finder.include_files(dll_path, Path("lib", dll_name))
-
-
 def load_tokenizers(finder: ModuleFinder, module: Module) -> None:
     """On Linux the tokenizers.libs directory is not copied."""
     if module.path is None:
@@ -698,17 +584,25 @@ def load_twisted_conch_ssh_transport(
     finder: ModuleFinder, module: Module
 ) -> None:
     """The twisted.conch.ssh.transport module uses __import__ builtin to
-    dynamically load different ciphers at runtime."""
+    dynamically load different ciphers at runtime.
+    """
     finder.include_package("Crypto.Cipher")
 
 
 def load_twitter(finder: ModuleFinder, module: Module) -> None:
     """The twitter module tries to load the simplejson, json and django.utils
     module in an attempt to locate any module that will implement the
-    necessary protocol; ignore these modules if they cannot be found."""
+    necessary protocol; ignore these modules if they cannot be found.
+    """
     module.ignore_names.update(["json", "simplejson", "django.utils"])
 
 
+def load_tzdata(finder: ModuleFinder, module: Module) -> None:
+    """The tzdata package requires its zone and timezone data."""
+    if module.in_file_system == 0:
+        finder.zip_include_files(module.file.parent, "tzdata")
+
+
 def load_uvloop(finder: ModuleFinder, module: Module) -> None:
     """The uvloop module implicitly loads an extension module."""
     finder.include_module("uvloop._noop")
@@ -716,7 +610,8 @@ def load_uvloop(finder: ModuleFinder, module: Module) -> None:
 
 def load_win32api(finder: ModuleFinder, module: Module) -> None:
     """The win32api module implicitly loads the pywintypes module; make sure
-    this happens."""
+    this happens.
+    """
     finder.exclude_dependent_files(module.file)
     finder.include_module("pywintypes")
 
@@ -724,13 +619,15 @@ def load_win32api(finder: ModuleFinder, module: Module) -> None:
 def load_win32com(finder: ModuleFinder, module: Module) -> None:
     """The win32com package manipulates its search path at runtime to include
     the sibling directory called win32comext; simulate that by changing the
-    search path in a similar fashion here."""
+    search path in a similar fashion here.
+    """
     module.path.append(module.file.parent.parent / "win32comext")
 
 
 def load_win32file(finder: ModuleFinder, module: Module) -> None:
     """The win32file module implicitly loads the pywintypes and win32timezone
-    module; make sure this happens."""
+    module; make sure this happens.
+    """
     finder.include_module("pywintypes")
     finder.include_module("win32timezone")
 
@@ -740,55 +637,30 @@ def load_wx_lib_pubsub_core(finder: ModuleFinder, module: Module) -> None:
     be done in a frozen application in the same way; modify the module
     search path here instead so that the right modules are found; note
     that this only works if the import of wx.lib.pubsub.setupkwargs
-    occurs first."""
+    occurs first.
+    """
     module.path.insert(0, module.file.parent / "kwargs")
 
 
-def load_Xlib_display(finder: ModuleFinder, module: Module) -> None:
-    """The Xlib.display module implicitly loads a number of extension modules;
-    make sure this happens."""
-    finder.include_module("Xlib.ext.xtest")
-    finder.include_module("Xlib.ext.shape")
-    finder.include_module("Xlib.ext.xinerama")
-    finder.include_module("Xlib.ext.record")
-    finder.include_module("Xlib.ext.composite")
-    finder.include_module("Xlib.ext.randr")
-
-
-def load_Xlib_support_connect(finder: ModuleFinder, module: Module) -> None:
-    """The Xlib.support.connect module implicitly loads a platform specific
-    module; make sure this happens."""
-    if sys.platform.split("-", maxsplit=1)[0] == "OpenVMS":
-        module_name = "vms_connect"
-    else:
-        module_name = "unix_connect"
-    finder.include_module(f"Xlib.support.{module_name}")
-
-
-def load_Xlib_XK(finder: ModuleFinder, module: Module) -> None:
-    """The Xlib.XK module implicitly loads some keysymdef modules; make sure
-    this happens."""
-    finder.include_module("Xlib.keysymdef.miscellany")
-    finder.include_module("Xlib.keysymdef.latin1")
-
-
 def load_xml_etree_cElementTree(finder: ModuleFinder, module: Module) -> None:
     """The xml.etree.cElementTree module implicitly loads the
-    xml.etree.ElementTree module; make sure this happens."""
+    xml.etree.ElementTree module; make sure this happens.
+    """
     finder.include_module("xml.etree.ElementTree")
 
 
 def load_yaml(finder: ModuleFinder, module: Module) -> None:
-    """PyYAML requires its metadata"""
+    """PyYAML requires its metadata."""
     module.update_distribution("PyYAML")
 
 
 def load_zmq(finder: ModuleFinder, module: Module) -> None:
     """The zmq package loads zmq.backend.cython dynamically and links
     dynamically to zmq.libzmq or shared lib. Tested in pyzmq 16.0.4 (py36),
-    19.0.2 (MSYS2 py39) up to 22.2.1 (from pip and from conda)."""
+    19.0.2 (MSYS2 py39) up to 22.2.1 (from pip and from conda).
+    """
     finder.include_package("zmq.backend.cython")
-    if WIN32:
+    if IS_WINDOWS or IS_MINGW:
         # For pyzmq 22 the libzmq dependencies are located in
         # site-packages/pyzmq.libs
         libzmq_folder = "pyzmq.libs"
@@ -796,64 +668,29 @@ def load_zmq(finder: ModuleFinder, module: Module) -> None:
         if libs_dir.exists():
             finder.include_files(libs_dir, Path("lib", libzmq_folder))
     # Include the bundled libzmq library, if it exists
-    try:
+    with suppress(ImportError):
         finder.include_module("zmq.libzmq")
-    except ImportError:
-        pass  # assume libzmq is not bundled
     finder.exclude_module("zmq.tests")
 
 
-def load_zoneinfo(finder: ModuleFinder, module: Module) -> None:
-    """The zoneinfo package requires timezone data, that
-    can be the in tzdata package, if installed."""
-    tzdata: Optional[Module] = None
-    source: Optional[Path] = None
-    try:
-        tzdata = finder.include_package("tzdata")
-        # store tzdata along with zoneinfo
-        tzdata.in_file_system = module.in_file_system
-    except ImportError:
-        zoneinfo = __import__(module.name, fromlist=["TZPATH"])
-        if zoneinfo.TZPATH:
-            for path in zoneinfo.TZPATH:
-                if path.endswith("zoneinfo"):
-                    source = Path(path)
-                    break
-        if source and source.is_dir():
-            # without tzdata, copy only zoneinfo directory
-            # in Linux: /usr/share/zoneinfo
-            target = Path("lib", "tzdata", "zoneinfo")
-            finder.include_files(source, target, copy_dependent_files=False)
-            finder.add_constant("PYTHONTZPATH", str(source))
-    if tzdata is None:
-        return
-    # when the tzdata exists, copy other files in this directory
-    source = tzdata.path[0]
-    target = Path("lib", "tzdata")
-    if tzdata.in_file_system >= 1:
-        finder.include_files(source, target, copy_dependent_files=False)
-    else:
-        finder.zip_include_files(source, "tzdata")
-
-
-load_backports_zoneinfo = load_zoneinfo
-
-
 def load_zope_component(finder: ModuleFinder, module: Module) -> None:
     """The zope.component package requires the presence of the pkg_resources
-    module but it uses a dynamic, not static import to do its work."""
+    module but it uses a dynamic, not static import to do its work.
+    """
     finder.include_module("pkg_resources")
 
 
 def missing_gdk(finder: ModuleFinder, caller: Module) -> None:
     """The gdk module is buried inside gtk so there is no need to concern
-    ourselves with an error saying that it cannot be found."""
+    ourselves with an error saying that it cannot be found.
+    """
     caller.ignore_names.add("gdk")
 
 
 def missing_ltihooks(finder: ModuleFinder, caller: Module) -> None:
-    """This module is not necessairly present so ignore it when it cannot be
-    found."""
+    """The ltihooks module is not necessairly present so ignore it when it
+    cannot be found.
+    """
     caller.ignore_names.add("ltihooks")
 
 
@@ -861,5 +698,5 @@ def missing_readline(finder: ModuleFinder, caller: Module) -> None:
     """The readline module is not normally present on Windows but it also may
     be so instead of excluding it completely, ignore it if it can't be found.
     """
-    if WIN32:
+    if IS_WINDOWS:
         caller.ignore_names.add("readline")
